'use strict';

var p = require('path');
// const printAST = require('ast-pretty-print')

module.exports = macrosPlugin;

function macrosPlugin() {
  return {
    name: 'macros',
    visitor: {
      ImportDeclaration(path, _ref) {
        var filename = _ref.file.opts.filename;

        var isMacros = looksLike(path, {
          node: {
            source: {
              value: function value(v) {
                return v.endsWith('.macros');
              }
            }
          }
        });
        if (!isMacros) {
          return;
        }
        var name = path.node.specifiers[0].local.name;
        var referencePaths = path.scope.getBinding(name).referencePaths;
        if (referencePaths && referencePaths.length) {
          var requirePath = p.join(p.dirname(filename), path.node.source.value);
          // eslint-disable-next-line import/no-dynamic-require
          var macros = require(requirePath);
          referencePaths.forEach(function (ref) {
            macros(ref.parentPath.get('quasi'));
          });
        }
        path.remove();
      }
    }
  };
}

function looksLike(a, b) {
  return a && b && Object.keys(b).every(function (bKey) {
    var bVal = b[bKey];
    var aVal = a[bKey];
    if (typeof bVal === 'function') {
      return bVal(aVal);
    }
    return isPrimitive(bVal) ? bVal === aVal : looksLike(aVal, bVal);
  });
}

function isPrimitive(val) {
  // eslint-disable-next-line
  return val == null || /^[sbn]/.test(typeof val);
}